from .core import BenchmarkId, IBenchmark
from .benchmarks.hibench import HiBench
from .benchmarks.mamul1 import MaMul1
from .util import Singleton
import logging
import argparse
from pathlib import Path
import json

__version__ = '0.0.1'


class BenchmarkFactory(metaclass=Singleton):

    def __init__(self):
        pass

    def create_benchmark(self, bench_id: BenchmarkId) -> IBenchmark:
        benchmark = {
            'hibench': HiBench(),
            'mamul1': MaMul1()
        }[bench_id]
        return benchmark


def main():
    """

    """
    logging.basicConfig(level=logging.DEBUG)

    example_text = '''example:

    %(prog)s --benchmark-id 'mamul1' --config '{"compiler_id": "gfortran", "matrix_size": 1024, "num_loops":10, "num_cores":2}' --results-dir /tmp/mamul1_out

    '''

    arg_parser = argparse.ArgumentParser(description='executes a benchmark in a cluster job environment', epilog=example_text, formatter_class=argparse.RawDescriptionHelpFormatter)
    arg_parser.add_argument('--benchmark-id', type=BenchmarkId, required=True, help='the benchmark id of the benchmark to perform (eg mamul1)')
    arg_parser.add_argument('--results-dir', type=Path, required=True, help='the root directory of the tree where the results of the benchmarks are stored (eg $GLOBAL_WORK_DIR/graffy/benchmarks/hibench)')
    arg_parser.add_argument('--config', type=str, default='cmake', help='the benchmark configuration in json format, eg {"compiler_id": "gfortran", "matrix_size": 1024}')

    args = arg_parser.parse_args()
    benchmark_id = BenchmarkId(args.benchmark_id)
    benchmark = BenchmarkFactory().create_benchmark(benchmark_id)
    benchmark_config = json.loads(args.config)
    benchmark.validate_config(benchmark_config)
    results_dir = args.results_dir
    benchmark.execute(benchmark_config, results_dir)
